# @(#)DN2Doc.tcl	1.2
#
# Code for displaying the documents that come back from the query.
#
# The window is basically just a textwin with scrollbars that will
# accept the text of the document and displays it.
#
# The namespace will be DN2Doc.  There will only be one interface routine.
# DN2Doc::Display $text_to_display

namespace eval DN2Doc {

    proc Display {canvas} {
	regsub -all {\.} $canvas {_} uniquifier
	set index [.dn2doc$uniquifier.upperframe.listframe.list curselection]
	set id [.dn2doc$uniquifier.upperframe.listframe.list get $index]
	set documentdata [getdocument $id]
	#InsertDocText $canvas $DN2Win::queryresults($id)
	InsertDocText $canvas $documentdata
    }

    proc InsertDocText {canvas displaytext} {
	regsub -all {\.} $canvas {_} uniquifier
	set textarea .dn2doc$uniquifier.upperframe.textframe.text
	$textarea delete 1.0 end
	$textarea insert end $displaytext
    }

    proc UpdateDocWin {canvas} {
	regsub -all {\.} $canvas {_} uniquifier
	set win .dn2doc$uniquifier
	if {[winfo exists .dn2doc$uniquifier]} {
	    # clear the scrolllist
	    # clear the textwin
	    # populate the scrolllist
	    $win.upperframe.textframe.text delete 1.0 end
	    $win.upperframe.listframe.list delete 0 end
	    foreach i [lsort $DN2Win::documentids] {
		$win.upperframe.listframe.list insert end $i
	    }
	}
    }

    proc CreateDocWin {canvas} {
	regsub -all {\.} $canvas {_} uniquifier
	if {![winfo exists .dn2doc$uniquifier]} {
	    # create the window
	    set w [toplevel .dn2doc$uniquifier]
	    wm title $w "Document"
	
	    set uf [frame $w.upperframe]
	    # create text widget with attached x and y scrollbars in a frame
	    set f [frame $uf.textframe]
	    text $f.text -xscrollcommand "$f.scrollx set" -yscrollcommand "$f.scrolly set" -setgrid true -wrap word
	    scrollbar $f.scrollx -orient horizon -command "$f.text xview"
	    scrollbar $f.scrolly -command "$f.text yview"
	    grid rowconfig $f 0 -weight 1
	    grid columnconfig $f 0 -weight 1
	    grid configure $f.text -in $f -row 0 -column 0 -sticky news
	    grid configure $f.scrollx -in $f -row 1 -column 0 -sticky news
	    grid configure $f.scrolly -in $f -row 0 -column 1 -sticky news
	    pack $f -side right -expand yes -fill both
	    
	    # create listbox on left side
	    set l [frame $uf.listframe]
	    scrollbar $l.scrolly -command "$l.list yview"
	    listbox $l.list -width 20 -selectmode single -yscroll "$l.scrolly set"
	    bind $l.list <Double-1> "DN2Doc::Display $canvas"
 
	    pack $l.list $l.scrolly -side left -fill y -expand 1
	    pack $l -side left -fill y -expand 1

	    # now pack the frame that the textframe and listframe are in
	    pack $uf -side top -expand yes -fill both

	    # create the dismiss button
	    button $w.dismissbutton -text "Dismiss" -command "DN2Doc::DestroyWin $canvas"
	    pack $w.dismissbutton -side bottom
	}
    }

    proc DestroyWin {canvas} {
	regsub -all {\.} $canvas {_} uniquifier
	if {[winfo exist .dn2doc$uniquifier]} {
	    destroy .dn2doc$uniquifier
	}
    }

}
