# %W%
# should contain all of the DN2 tree interface code.

#proc CreateTree {treeid dn2data} {
#    # loads the dn2data into the treedata structure
#
#    PopulateTreeDataWithDN2 $treeid $dn2data
#    TreeData::MakeViewableTree $treeid
#}

#proc NewTree {treeid} {
#    # clear out the old tree
#    # start a new one with a single DN2 node
#    TreeData::makenewroot $treeid
#
#}

#proc ClearTree{} {
#
#}

proc PopulateTreeWithDN2 {treeid data} {
    # adds the root node then loops through the subnodes

    set typeofnode [lindex $data 0]
    set attributes [lindex $data 1]
    set subnodes [lindex $data 2]

    # putting in root node
    set rootid [TreeData::makenewroot $treeid $typeofnode $attributes {}]

    for {set i 0} { $i < [llength $subnodes]} {incr i} {
	AddNodeToTreeData $treeid $rootid [lindex $subnodes $i]
    }
}


proc AddNodeToTreeData {treeid parentid nodedata} {
    set type_of_node [lindex $nodedata 0]
    set attriblist [lindex $nodedata 1]
    set subnodes [lindex $nodedata 2]

    # because the tcl format contains the text data as an 
    # attribute with name value, find it and remove it from 
    # the list.
    set cleanattribs {}
    set savedvalue ""
    foreach i $attriblist {
	if {[string compare "value" [lindex $i 0]] == 0} {
	    # found value attribute
	    set savedvalue [lindex $i 1]
	} else {
	    lappend cleanattribs $i
	}
    }
    
    # start of hack to place text dump formatted data into textdata
    # create textwin, insert text into it, then dump it back out.
    set tmpvar {}
    if {[string length $savedvalue] > 0} {
	text .text
	.text insert end $savedvalue
	set tmpvar [.text dump -text 1.0 end]
	destroy .text
    }
    # end of hack

    # create new node and make the rightmost child of the parent
    set newid [TreeData::makenewnode $treeid $type_of_node $cleanattribs $tmpvar]
    TreeData::insert_rightmostchild $parentid $newid

    # now insert all this node's children
    for {set i 0} {$i < [llength $subnodes]} {incr i} {
	AddNodeToTreeData $treeid $newid [lindex $subnodes $i]
    }
}

proc MakeViewableTree {treeid} {
    # given the treeid this will create a
    # displayable tree

    set tree_wgt [TreeData::getdisplaytreeid $treeid]
    $tree_wgt prune {}
    set canvas [winfo parent $tree_wgt]
    
    # starting at the root get the id of the node, create
    # some canvas widgets with the id as a tag, add this to
    # the tree widget,  do this for each of the children 
    # of the current node.  Just realized that I cannot use
    # integers as an index since you cannot use an integer as
    # a tag.  I am going to leave the index into treedata an
    # integer but the tag I will use is $id + :id.  I can strip
    # out the $id value with "regexp ^\[0-9\] $idvar index".
    # 31 July: the previous statements just changed.  The index in
    # no longer an integer.  I don't think I need to use the :id 
    # structure any longer.
    
    make_display_node $canvas $tree_wgt {} [TreeData::getrootid $treeid]

}

proc make_display_node {canvas tree_wgt wgt_parent id} {
    # actual recursive routine which insert the tree data into
    # the tree widget
    
    # draw text
    # draw box around it
    # add tags
    # create line
    # add node to tree_wgt
    # run through the children calling display node for each
    #  one of them.

    # I can put some sort of nodetype dependance here so that I can control what gets displayed.
    if {[string compare [TreeData::getnodetype $id] ANN-ATTR] == 0} {
	# This subtree can be collapsed into a single supernode
	collapse_ANN-ATTR $canvas $tree_wgt $wgt_parent $id
	return
    } elseif {[string compare [TreeData::getnodetype $id] SE-APP] == 0} {
	# This subtree can be collapsed into a single supernode
	collapse_SE-APP $canvas $tree_wgt $wgt_parent $id
	return
    } elseif {[string compare [TreeData::getnodetype $id] TEXT] == 0} {
	# if TEXT node then make textstring the first 10 characters
	set textdata [TreeData::gettextdata $id]
	set textstring "\""
	for {set i 1} {$i < [llength $textdata]} {incr i 3} {
	    append textstring [lindex $textdata $i]
	}
	set displaytext [string trimright [string range $textstring 0 9]]
	if {[string length [string trimright $textstring]] > [string length $displaytext]} {
	    append displaytext "...\""
	} else {
	    append displaytext "\""
	}
	
    } else {
	set displaytext [TreeData::getnodetype $id]
    }
 
    make_box_and_text $canvas $tree_wgt $wgt_parent $id $displaytext 0

    if {[TreeData::getchildid $id] != 0} {
	# there are children so them to the tree widget
	set current [TreeData::getchildid $id]
	make_display_node $canvas $tree_wgt $id $current
	while {[TreeData::getsiblingid $current] != 0} {
	    make_display_node $canvas $tree_wgt $id [TreeData::getsiblingid $current]
	    set current [TreeData::getsiblingid $current]
	}
    }
}

proc make_box_and_text {canvas tree_wgt parent id textstring compositep} {

    set compositestr noncomposite
    if {$compositep == 1} {
	set compositestr composite
    }
    set textid [$canvas create text 0 0 -text $textstring -tags [list [TreeData::getnodetype $id] $id node $compositestr]]

    scan [$canvas bbox $textid] "%d %d %d %d" x1 y1 x2 y2
    set x1 [expr $x1 - 5]
    set y1 [expr $y1 - 5]
    set x2 [expr $x2 + 5]
    set y2 [expr $y2 + 5]

    set boxid [$canvas create rectangle $x1 $y1 $x2 $y2 -fill grey -tags [list [TreeData::getnodetype $id] $id node $compositestr]]
    $canvas lower $boxid
    $canvas addtag $id:txt withtag $textid
    $canvas addtag $id:box withtag $boxid
    set lineid [$canvas create line 0 0 0 0]
    $tree_wgt addlink $parent $id $lineid

}
    
proc collapse_ANN-ATTR {canvas tree_wgt parent_wgt id} {
    # collapse the ANN-ATTR subtree into a single node
    # which will have the attributes of the subnodes

    # creating an array which will contain the data from the possible subnodes of
    # an ANN-ATTR node.
    # This array could be used then for the supernode
    array set ann_attr {{ANN-TYPE {}} {ATTR-NAME {}} {ATTR-TYPE {}} {ATTR-VALUE {}}}
    set current [TreeData::getchildid $id]
    if {$current != 0} {
	set ann_attr([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	set current [TreeData::getsiblingid $current]
	while {$current != 0} {
	    set ann_attr([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	    set current [TreeData::getsiblingid $current]
	}
    }

    set displaytext "ANN-ATTR"
    if {[info exists ann_attr(ANN-TYPE)]} {
	append displaytext "\nANN-TYPE:\t"
	set textstring {}
	for {set i 1} {$i < [llength $ann_attr(ANN-TYPE)]} {incr i 3} {
	    append textstring [lindex $ann_attr(ANN-TYPE) $i]
	}
	set textstring [string trimright $textstring]
	if {[string length $textstring] > 10} {
	    set textstring [string range $textstring 0 6]
	    append textstring "..."
	}
	append displaytext $textstring
	set added_one 1
    }
    if {[info exists ann_attr(ATTR-TYPE)]} {
	append displaytext "\nATTR-TYPE:\t"
	set textstring {}
	for {set i 1} {$i < [llength $ann_attr(ATTR-TYPE)]} {incr i 3} {
	    append textstring [lindex $ann_attr(ATTR-TYPE) $i]
	}
	set textstring [string trimright $textstring]
	if {[string length $textstring] > 10} {
	    set textstring [string range $textstring 0 6]
	    append textstring "..."
	}
	append displaytext $textstring
	set added_one 1
    }
    if {[info exists ann_attr(ATTR-NAME)]} {
	append displaytext "\nATTR-NAME:\t"
	set textstring {}
	for {set i 1} {$i < [llength $ann_attr(ATTR-NAME)]} {incr i 3} {
	    append textstring [lindex $ann_attr(ATTR-NAME) $i]
	}
	set textstring [string trimright $textstring]
	if {[string length $textstring] > 10} {
	    set textstring [string range $textstring 0 6]
	    append textstring "..."
	}
	append displaytext $textstring
	set added_one 1
    }
    if {[info exists ann_attr(ATTR-VALUE)]} {
	append displaytext "\nATTR-VALUE:\t"
	set textstring {}
	for {set i 1} {$i < [llength $ann_attr(ATTR-VALUE)]} {incr i 3} {
	    append textstring [lindex $ann_attr(ATTR-VALUE) $i]
	}
	set textstring [string trimright $textstring]
	if {[string length $textstring] > 10} {
	    set textstring [string range $textstring 0 6]
	    append textstring "..."
	}
	append displaytext $textstring
	set added_one 1
    }


    make_box_and_text $canvas $tree_wgt $parent_wgt $id $displaytext 1
    
	    
}

proc collapse_FULL-TERM {canvas tree_wgt parent_wgt id} {
    # collapse the FULL-TERM subtree into a single node
    # tougher than ANN-ATTR since this subtree can have
    # 0 or more ann-attr's

    set displaytext "FULL-TERM comp"
    make_box_and_text $canvas $tree_wgt $parent_wgt $id $displaytext 1

}

proc collapse_SE-APP {canvas tree_wgt parent_wgt id} {
    # collapse the SE-APP subtree into a single node
    # not too bad since all of the subnodes except for
    # SE-APP-MATCHES are 0 or 1, but Matches is 0 or more.

    set displaytext "SE-APP"
    make_box_and_text $canvas $tree_wgt $parent_wgt $id $displaytext 1

}

