# %W%
# Creates a window with a text widget displaying the DN2 representing the tree.

namespace eval DN2Win {

    set documentids {}
    set searchengine {}

proc PopulateDN2Win {canvas textwin} {
    DN2Construct::GetDN2String $canvas dn2string
    $textwin delete 1.0 end
    $textwin insert end $dn2string
}

proc GetDN2String {textwin retvar} {
    upvar 1 $retvar dn2str
    set textstring [$textwin dump -text 1.0 end]
    set strlen [llength $textstring]
    for {set i 1} { $i < $strlen} {set i [expr $i+3]} {
	append dn2str [lindex $textstring $i]
    }
}

proc ParseDN2 {textwin} {
    # get the string containing the DN2 to parse
    # send it throught parsedn2
    # display result
    GetDN2String $textwin dn2string
    set retval [parsedn2 $dn2string returneddn2]
    if {[string compare $retval "0"] != 0} {
	set linenum [lindex [split $retval :] 1]
	$textwin tag add error $linenum.0 $linenum.end
	$textwin tag configure error -foreground red
	tk_messageBox -type ok -message "Error in Line $linenum\n$retval"
	$textwin tag delete error
    } else {
	tk_messageBox -type ok -message "Successfully Parsed."
    }
}

proc Search {textwin} {
    # takes the DN2 from the textwin and 
    # sends it off to the search engine
    # the returned data are:
    #   the search engine modified dn2
    #   an array of document hits

    # which search engine?
    puts $DN2Win::searchengine
    #

    GetDN2String $textwin dn2string
    set retval [querydn2 $DN2Win::searchengine $dn2string returneddn2 DN2Win::documentids]
    if {$retval != 1} {
	return
    }
    # display the returned dn2 in a textwindow and
    # pop up the list of document ids in a scrolllist in another
    #  window.  Selecting a doc id pops up the document.

    DN2Doc::CreateDocWin $textwin
    DN2Doc::UpdateDocWin $textwin
    
    # create a window in which to put the returneddn2

    CreateModDN2Win $textwin $returneddn2

}

proc CreateModDN2Win {textwin moddn2} {
    
    catch {destroy .xxxwinxxx}
    set w [toplevel .xxxwinxxx]

    set tview [frame $w.treeview]

    scrollbar $tview.hscroll -orient horiz -command "$tview.canv xview"
    scrollbar $tview.vscroll -command "$tview.canv yview"
    set canvas [canvas $tview.canv -relief sunken -borderwidth 2 -scrollregion {0i 0i 6i 6i} \
	    -width 3i -height 3i \
	    -xscrollcommand "$tview.hscroll set" \
	    -yscrollcommand "$tview.vscroll set"]
    
    # create tree in the canvas
    set tree [tree $tview.canv.t -layout horizontal]
    
    # initialize the treedata
    TreeData::init $canvas
    TreeData::setdisplaytreeid $canvas $tree
    
    # put the frame components into a grid
    grid $tview.canv -padx 1 -pady 1 -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky news
    grid $tview.hscroll -padx 1 -pady 1 -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky news
    grid $tview.vscroll -padx 1 -pady 1 -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky news
    grid rowconfig $tview 0 -weight 1 -minsize 0
    grid columnconfig $tview 0 -weight 1 -minsize 0
    pack  $tview -side top -fill both -expand yes
    
    puts "Creating menu now."
    menu $w.menu -tearoff 0
    
    set m [menu $w.menu.file -tearoff 0]
    $w.menu add cascade -label "View" -menu $m -underline 0
    $m add separator
    $m add command -label "View Attributes" -command "ViewAttribs::CreateViewAttribWin $canvas"
    $m add command -label "Dismiss" -command "destroy $w"
    set m [menu $w.menu.nodes -tearoff 0]
    $w configure -menu $w.menu
    
    setup_bindings $canvas $tree

    # convert returneddn2 into the internal tclformat
    set dn2error [parsedn2 -tcl $moddn2 tcldn2]
    DN2Application::NewTree $canvas
    DN2Application::ClearTree $canvas
    DN2Application::CreateTree $canvas [lindex $tcldn2 0]
    wm title $w "Modified DN2"

}

proc ApplyDN2Changes {canvas text} {
    # parse dn2 for errors return if there are any
    # if successful then delete old dn2 tree
    # recreate dn2 tree with text in the text widget
    
    GetDN2String $text dn2text
    set dn2errors [parsedn2 -tcl $dn2text tcltext]
    if {$dn2errors != 0} {
	tk_messageBox -type ok -message "DN2 has errors!"
	return
    }
    DN2Application::ClearTree $canvas
    DN2Application::CreateTree $canvas [lindex $tcltext 0]
}

proc CreateDN2Win {canvas tree} {
    regsub -all {\.} $canvas {_} uniquifier
    if {![winfo exists .dn2win$uniquifier]} {
	# window doesn't exist create it
	set w [toplevel .dn2win$uniquifier]

	wm title $w "DN2 Representation"
	
	# create text widget with attached x and y scrollbars in a frame
	set f [frame $w.textframe]
	text $f.text -xscrollcommand "$f.scrollx set" -yscrollcommand "$f.scrolly set" -setgrid true -wrap none
	$f.text tag configure error -relief raised
	scrollbar $f.scrollx -orient horizon -command "$f.text xview"
	scrollbar $f.scrolly -command "$f.text yview"
	grid rowconfig $f 0 -weight 1
	grid columnconfig $f 0 -weight 1
	grid configure $f.text -in $f -row 0 -column 0 -sticky news
	grid configure $f.scrollx -in $f -row 1 -column 0 -sticky news
	grid configure $f.scrolly -in $f -row 0 -column 1 -sticky news
	pack $f -side top -expand yes -fill both
	
	# create the buttons
	button $w.sendbutton -text "Search" -command "DN2Win::Search $f.text"
	CreateEngineMenu $w
	#tk_optionMenu $w.enginemenu DN2Win::searchengine a b c
	button $w.parsebutton -text "Parse" -command "DN2Win::ParseDN2 $f.text"
	button $w.applybutton -text "Apply" -command "DN2Win::ApplyDN2Changes $canvas $f.text"
	button $w.dismissbutton -text "Dismiss" -command "DN2Win::DestroyWin $canvas"
	pack $w.sendbutton $w.enginemenu $w.parsebutton $w.applybutton $w.dismissbutton -side left
    } else {
	raise .dn2win$uniquifier
	set f .dn2win$uniquifier.textframe
    }

    # fill text widget with the dn2
    PopulateDN2Win $canvas $f.text
}

proc CreateEngineMenu {window} {
    set infile [open coll_list]
    set collection [read $infile]
    close $infile
    set firstchoice ""
    set command "tk_optionMenu $window.enginemenu DN2Win::searchengine "
    foreach i [split $collection \n] {
	if {[llength $i] > 0} {
	    append command "[lindex [split $i \t] 0] "
	    if {[string compare $firstchoice ""] == 0} {
		# set firstchoice to first item in option menu
		set firstchoice [lindex [split $i \t] 0]
	    }
	}
    }
    set DN2Win::searchengine $firstchoice
    eval $command
}



proc DestroyWin {canvas} {
    regsub -all {\.} $canvas {_} uniquifier
    if {[winfo exists .dn2win$uniquifier]} {
	destroy .dn2win$uniquifier
    }
}

}
