# @(#)add_attribs.tcl	1.4
# The routines needed for creating, manipulating, and destroying the AddAttribs window
# 
#

namespace eval AddAttrib {

proc AddNewAttrib {win canvas} {
    # Adds the attribute name selected from the addattribwin to the list of
    # of attributes for a node.
    
    set newattrib [$win.f1.lbox get [$win.f1.lbox curselection]]
    set nodetags [$canvas gettags selected]
    set nodeid [lindex $nodetags 1]
    set nodetype [TreeData::getnodetype $nodeid]
    set attriblist [TreeData::getattriblist $nodeid]
    set textid_in_canvas [$canvas find withtag selected]
    set boxid_in_canvas [$canvas find withtag $nodeid:box]

    # if node is ANN-ATTR then we need to do special stuff
    if {[string compare $nodetype ANN-ATTR] == 0} {
	AddANN-ATTR $canvas $nodeid $newattrib
    } else {
	# check for duplicate and do nothing if newattrib is already in attriblist
	if {[lsearch -regexp $attriblist $newattrib] != -1} {
	    return
	}
	
	# an empty attriblist contains 1 blank list which must be removed prior to
	# appending a newattrib
	if {[llength [lindex $attriblist 0]] == 0} {
	    set attriblist {}
	}
	lappend attriblist [list $newattrib {}]

	TreeData::setattriblist $nodeid $attriblist
    }
}

proc AddANN-ATTR {treeid nodeid newsubnode} {
    # determine if newsubnode is a duplicate then don't add it.
    # create a new subnode and add it to nodeid
    
    set isduplicate 0
    set current [TreeData::getchildid $nodeid]
    if {$current != 0} {
	if {[string compare $newsubnode [TreeData::getnodetype $current]] == 0} {
	    return
	}
	while {!$isduplicate && [TreeData::getsiblingid $current] != 0} {
	    set current [TreeData::getsiblingid $current]
	    if {[string compare $newsubnode [TreeData::getnodetype $current]] == 0} {
		return
	    }
	}
    }

    # the new subnode is not a duplicate so add it

    set newid [TreeData::makenewnode $treeid $newsubnode {} {}]
    TreeData::insert_rightmostchild $nodeid $newid
}

proc CreateAddAttribWin {canvas} {

    regsub -all {\.} $canvas {_} uniquifier
    set win .addattribwin$uniquifier
    catch {destroy $win}
    global attributes_of_type
    toplevel $win
    grab set $win

    # Create window with widgets
    set f1 [frame $win.f1]
    listbox $f1.lbox -yscroll "$f1.scroll set" -setgrid 1 -height 5 -selectmode single
    scrollbar $f1.scroll -command "$f1.lbox yview"
    pack $f1.lbox -side left -expand y -fill both
    pack $f1.scroll -side right -expand y -fill y
    pack $f1 -side top -expand y
    set f2 [frame $win.f2]
    button $f2.okbut -text "Ok" -command "AddAttrib::AddNewAttrib $win $canvas; AddAttrib::DestroyWin $canvas; ViewAttribs::UpdateWindow $canvas"
    button $f2.cancelbut -text "Cancel" -command "AddAttrib::DestroyWin $canvas"
    pack $f2.okbut $f2.cancelbut -side left -fill x -expand 1
    pack $f2

    # Populate listbox
    set tags [$canvas gettags [$canvas find withtag selected]]
    set nodetype [lindex $tags 0]
    foreach attrib $attributes_of_type($nodetype) {
	$f1.lbox insert end $attrib
    }
}

proc DestroyWin {canvas} {

    regsub -all {\.} $canvas {_} uniquifier
    set win .addattribwin$uniquifier
    grab release $win
    if {[winfo exists $win]} {
	destroy $win
    }
}

}


