# @(#)add_sub_node.tcl	1.10
# Contains the procedures which handle creating the AddSubNodeWin and the callbacks.

namespace eval AddSubNode {

    proc AddTextSubNodeTo {canvas id} {
	# add a text to node id
	global allowable_subnode
	set tree [TreeData::getdisplaytreeid $canvas]
	if {[string length [$tree child {}]] != 0} {
	    # there are nodes
	    if { [llength [$canvas find withtag $id]] == 0 } {
		return
	    }
	    set nodeid [lindex [$canvas gettags [$canvas find withtag $id]] 1]
	    # check that textnode is allowed as child
	    set typeofparent [TreeData::getnodetype $nodeid]
	    if { [lsearch -exact $allowable_subnode($typeofparent) TEXT] != -1} {
		AddNodeToTreeData $canvas $nodeid [list TEXT {{}} {}]
		MakeViewableTree $canvas
	    } else {
		tk_messageBox -type ok -message "Unable to add TEXT node to $typeofparent"
	    }
	}
    }


proc AddSubNodeTo {addwin canvas id} {
    # Bring up a modal window for adding attributes
    # then insert new node beneath node with id

    set list_of_attribs ""
    set tree [TreeData::getdisplaytreeid $canvas]
    # check to see if there are any nodes at all
    if {[string length [$tree child {}]] == 0 } {
	# there are no nodes so put new node onto the root
	# still have some work to here
	set nodetag {}
	return
    } else {
	# if a node with $id doesn't exist, leave
	if { [llength [$canvas find withtag $id]] == 0 } {
	    return
	}
	set nodeid [lindex [$canvas gettags [$canvas find withtag $id]] 1]
	puts "adding subnode to $id --> $nodeid"
	set typeofnode [$addwin.listframe1.lbox get active]
	# get list of selected attributes from the other list
	foreach i [$addwin.listframe2.lbox curselection] {
	    lappend list_of_attribs [list [$addwin.listframe2.lbox get $i] {}]
	}
    }

    if {[llength $list_of_attribs] == 0} {
	set list_of_attribs {{}}
    }

    if {[string compare $typeofnode "ANN-ATTR"] == 0} {
	# special node: the chosen attributes are really subnodes
	# so convert the list_of_attribs into a list_of_subnodes
	set list_of_subnodes {}
	for {set i 0} {$i < [llength $list_of_attribs]} {incr i} {
	    if {[llength [lindex $list_of_attribs $i]] > 0} {
		lappend list_of_subnodes [list [lindex [lindex $list_of_attribs $i] 0] {{}} {}]
	    }
	}
	AddNodeToTreeData $canvas $nodeid [list $typeofnode {{}} $list_of_subnodes]
    } else {
	set data [list $typeofnode $list_of_attribs {}]
	puts "Creating node of type $typeofnode"
	puts "with tags: $data"
	AddNodeToTreeData $canvas $nodeid $data
    }
    MakeViewableTree $canvas
}

proc fill_lbox {lbox data} {
    foreach i $data {
	$lbox insert end $i
    }
}

# Updates the listbox containing the list of node attributes
proc UpdateAttribListbox {w x y} {
    puts "Entered UpdateAttribListbox"
    global attributes_of_type
    global selectedtype
    set selectedtype [$w.listframe1.lbox get @$x,$y]
    puts "selected type was $selectedtype"
    $w.listframe2.lbox delete 0 end
    foreach i $attributes_of_type($selectedtype) {
	$w.listframe2.lbox insert end $i
    }
    # clear out the selected variables
    
}

proc UpdateWindow {canvas} {
    # fills the first listbox with the subnodes that are allowed to follow
    # the selected node.
    # If there is not a selected node then clear the listboxes
    global allowable_subnode
    regsub -all {\.} $canvas {_} uniquifier
    set w .addsubnode$uniquifier
    if {[winfo exists $w]} {
	if {[llength [$canvas find withtag selected]] != 0} {
	    # fill the first lbox with data
	    fill_lbox $w.listframe1.lbox $allowable_subnode([lindex [$canvas gettags selected] 0])
	} else {
	    # see if there are any nodes
	    # clear the lbox if there are nodes
	    $w.listframe1.lbox delete 0 end
	    $w.listframe2.lbox delete 0 end
	}
    }
}

# Called when the the addnode button is pressed on the main canvas
# creates a new window containing the types of nodes that can be added to the selected not
proc CreateAddSubNodeWin {canvas} {
    global node_types
    global allowable_subnode
    global attributes_of_type

    regsub -all {\.} $canvas {_} uniquifier
    if {[winfo exists .addsubnode$uniquifier]} {
	# update the existing window
	set w .addsubnode$uniquifier
    } else {
	# create new window
	set w [toplevel .addsubnode$uniquifier]
	# removing the grab so the window is now not modal
	#grab $w
	wm title $w "Add a Subnode"
    
	label $w.title -text "Select the type of the node then select attributes"
	pack $w.title -side top

	# Frame in which to put the node types listbox and scrollbar
	frame $w.listframe1
	listbox $w.listframe1.lbox -yscroll "$w.listframe1.scroll set" -setgrid 1 -height 3 -selectmode single
	scrollbar $w.listframe1.scroll -command "$w.listframe1.lbox yview"
	bind $w.listframe1.lbox <Button-1> "AddSubNode::UpdateAttribListbox $w %x %y"
	pack $w.listframe1.scroll -side right -fill y -expand yes
	pack $w.listframe1.lbox -side left -expand 1 -fill both -expand yes
	pack $w.listframe1 -expand yes -side top

	# Frame in which to put the attribute listbox and scrollbar
	frame $w.listframe2
	scrollbar $w.listframe2.scroll -command "$w.listframe2.lbox yview"
	listbox $w.listframe2.lbox -yscrollcommand "$w.listframe2.scroll set" -height 3 -selectmode multiple
	pack $w.listframe2.scroll -side right -fill both -expand yes
	pack $w.listframe2.lbox -side left -fill both -expand yes
	pack $w.listframe2 -expand yes

	frame $w.buttonframe
	button $w.buttonframe.okbut -text "Ok" -command "AddSubNode::AddSubNodeTo $w $canvas selected; AddSubNode::DestroyWin $canvas"
	button $w.buttonframe.cancelbut -text "Cancel" -command "AddSubNode::DestroyWin $canvas"
	pack $w.buttonframe.okbut $w.buttonframe.cancelbut -side left -fill x -expand 1
	pack $w.buttonframe -side bottom -fill x  -expand 0
    }

    UpdateWindow $canvas
}


proc DestroyWin {canvas} {
    regsub -all {\.} $canvas {_} uniquifier
    set win .addsubnode$uniquifier
    if {[winfo exists $win]} {
	destroy $win
    }
}

}
