# @(#)globalvars.tcl 1.7
# contains the global variables needed for the dn2tree 
# node_types is a list of the types of nodes allowed
# attributes_of_nodes is an a-list indexed by node_type and whose value is a list
#    of the allowed attributes for that node_type

set node_types {DN2 SE-APP SE-APP-HELP SE-APP-EXPL SE-APP-DOCFREQ SE-APP-COLLFREQ SE-APP-INTERNAL-ID SE-APP-MATCHES APP-SE CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM INFO-NEED DOCID DOC-COLLECTION B-AND B-OR B-AND-NOT DOCID-REL TEST-REL ANN-ATTR ANN-TYPE ATTR-NAME ATTR-TYP ATTR-VALUE MERGE-INFO COMMENT}

set attributes_of_type(DN2) {ID OUTPUT-QUERY OUTPUT-DOCS}
set attributes_of_type(SE-APP) {}
set attributes_of_type(SEAP-HLP) {}
set attributes_of_type(SE-APP-EXPL) {}
set attributes_of_type(SE-APP-DOCFREQ) {}
set attributes_of_type(SE-APP-COLLFREQ) {}
set attributes_of_type(SE-APP-INTERNAL-ID) {}
set attributes_of_type(SE-APP-MATCHES) {SPAN-START SPAN-END WEIGHT}
set attributes_of_type(APP-SE) {NUMBER-TO-RETRIEVE MIN-THRESHOLD MAX-THRESHOLD REFORMULATION  EXPANSION  STEMMING HELP EXPLAIN MATCHES DOCFREQ COLLFREQ}
set attributes_of_type(CONTEXT) {DISTANCE ORDERED}
set attributes_of_type(FULL-TERM) {WEIGHT}
set attributes_of_type(TEXT) {}
set attributes_of_type(INDEPENDENT) {WEIGHT}
set attributes_of_type(AND) {WEIGHT EXACT FUZZY}
set attributes_of_type(OR) {WEIGHT EXACT FUZZY}
set attributes_of_type(AND-NOT) {WEIGHT EXACT FUZZY}
set attributes_of_type(HEAD-RELATION) {HR-TYPE WEIGHT}
set attributes_of_type(OTHER-OPER) {WEIGHT}
set attributes_of_type(INFO-NEED) {WEIGHT}
set attributes_of_type(DOCID) {}
set attributes_of_type(DOC-COLLECTION) {}
set attributes_of_type(B-AND) {}
set attributes_of_type(B-OR) {}
set attributes_of_type(B-AND-NOT) {}
set attributes_of_type(RESTRICT-SET) {}
set attributes_of_type(FEEDBACK-INFO) {}
set attributes_of_type(DOCID-REL) {REL SPAN-START SPAN-END}
set attributes_of_type(TEXT-REL) {REL}
set attributes_of_type(COMMENT) {}
# ANN-ATTR does not really have attributes.  This is a way for the add_attribs code
# to display the possible subnodes of ANN-ATTR.
set attributes_of_type(ANN-ATTR) {ANN-TYPE ATTR-NAME ATTR-TYPE ATTR-VALUE}
set attributes_of_type(ANN-TYPE) {}
set attributes_of_type(ATTR-NAME) {}
set attributes_of_type(ATTR-TYPE) {}
set attributes_of_type(ATTR-VALUE) {}
set attributes_of_type(MERGE-INFO) {MERGETYPE}


# Contains the allowable subnodes for the currently selected node
# The array is indexed by the nodetype
set allowable_subnode(DN2) {SE-APP APP-SE CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM INFO-NEED COMMENT}
set allowable_subnode(SE-APP) {SE-APP-HELP SE-APP-EXPL SE-APP-DOCFREQ SE-APP-COLLFREQ SE-APP-INTERNAL-ID SE-APP-MATCHES}
set allowable_subnode(APP-SE) {}
set allowable_subnode(CONTEXT) {ANN-ATTR}
set allowable_subnode(FULL-TERM) {APP-SE SE-APP TEXT ANN-ATTR COMMENT}
set allowable_subnode(TEXT) {}
set allowable_subnode(INDEPENDENT) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(AND) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(OR) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(AND-NOT) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(HEAD-RELATION) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(OTHER-OPER) {APP-SE SE-APP CONTEXT AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(INFO-NEED) {DOC-COLLECTION RESTRICT-SET FEEDBACK-INFO AND OR AND-NOT HEAD-RELATION OTHER-OPER INDEPENDENT TEXT FULL-TERM COMMENT}
set allowable_subnode(DOCID) {}
set allowable_subnode(DOC-COLLECTION) {}
set allowable_subnode(B-AND) {B-AND B-OR B-AND-NOT TEXT FULL-TERM COMMENT}
set allowable_subnode(B-OR) {B-AND B-OR B-AND-NOT TEXT FULL-TERM COMMENT}
set allowable_subnode(B-AND-NOT) {B-AND B-OR B-AND-NOT TEXT FULL-TERM COMMENT}
set allowable_subnode(RESTRICT-SET) {B-AND B-OR B-AND-NOT DOCID COMMENT}
set allowable_subnode(FEEDBACK-INFO) {DOCID-REL TEXT-REL COMMENT}
set allowable_subnode(DOCID-REL) {}
set allowable_subnode(TEXT-REL) {}
set allowable_subnode(COMMENT) {}
set allowable_subnode(ANN-ATTR) {ANN-TYPE ATTR-NAME ATTR-TYPE ATTR-VALUE}
set allowable_subnode(ANN-TYPE) {}
set allowable_subnode(ATTR-NAME) {}
set allowable_subnode(ATTR-TYPE) {}
set allowable_subnode(ATTR-VALUE) {}
set allowable_subnode(MERGE-INFO) {}


# contains the list of nodetypes that have the value attribute
#  this means that these nodes have a data content that others do not
set nodetypes_with_value {SE-APP SE-APP-HELP SE-APP-DOCFREQ SE-APP-COLLFREQ SE-APP-INTERNAL-ID SE-APP-MATCHES TEXT COMMENT DOCID DOCID-REL DOC-COLLECTION TEXT-REL ANN-TYPE ATTR-NAME ATTR-TYPE ATTR-VALUE MERGE-INFO}

 
# contains the type of the currently selected node
set selectedtype {}

# contains the data of the qtd-text node
# the array is indexed by the canvasid along with the node id of the qtd-text node
array set qtd_text_value {}

# contains the introductory text for construction of a dn2
set DN2_header "<!DOCTYPE DN2 PUBLIC \"-//TIPSTER//DTD DN2//EN\">\n" 

