# @(#)node_actions.tcl 1.5
# collection of procedures that operate on nodes
#

proc CleanQtdTextArray {canvas tree id} {
    # if the node with id $id is a qtd-text type
    #  it clears out the qtd_text_value element associated with it
    #  loops through the children of the node calling CleanQtdTextArray on each one

    global qtd_text_value
    if { [lsearch -exact [array names qtd_text_value *] $canvas.$id] != -1} {
	# current node has an entry in the array
	unset qtd_text_value($canvas.$id)
    }
    foreach i [$tree subnodes $id] {
	CleanQtdTextArray $canvas $tree $i
    }
}

proc PruneNode {canvas tree id} {
    # prune selected node (deletes all subnodes)
    # keeping selected node
    # - must check for qtd-text nodes so that the qtd_text_array can be kept uptodate.
    if { [llength [$canvas find withtag $id]] == 0 } {
	return
    }
    set nodetag [lindex [$canvas gettags [$canvas find withtag $id]] 1]
    # since PruneNode leaves the selected node don't remove it's qtd_text_value entry
    # just do it's children
    # foreach i [$tree subnodes $nodetag] {
	#CleanQtdTextArray $canvas $tree $i
	
    # }
    puts "pruning node $id --> $nodetag"
    $tree prune $nodetag
    TreeData::prune_node $nodetag
}

proc RemoveNode {canvas tree id} {
    # deletes selected node and all beneath it
    # - must check for qtd-text nodes so that the qtd_text_array can be kept uptodate.
    if { [llength [$canvas find withtag $id]] == 0 } {
	return
    }
    set nodetag [lindex [$canvas gettags [$canvas find withtag $id]] 1]
    # since RemoveNode removes the selected node and its children just call
    # the CleanQtdTextArray on the selected node.
    CleanQtdTextArray $canvas $tree $nodetag
    puts "deleting node $id --> $nodetag"

    TreeData::remove_node $nodetag
    $tree rmlink $nodetag
    DN2Application::UpdateWindows $canvas
}

