# @(#)view_attribs.tcl	1.5
# this files will contain the procedures for creating and altering node
# attributes.  I need to add the code to allow for additions to the 
# list of attributes.  Meaning whole new attributes not just changing the value
# of an existing one.


namespace eval ViewAttribs {

proc EditAttribs {canvas win} {
    # makes the changes to the attribute list when the ok button has been pressed
    # Have to add specific code to handle the qtd-text value attribute due to the
    # creation of a text window.

    set id_selected [$canvas find withtag selected]
    if {[lsearch [$canvas gettags [$canvas find withtag selected]] noncomposite] == -1} {
	EditCompositeAttribs $canvas $win
    } else {
	EditNonCompositeAttribs $canvas $win
    }
    # update the displayed tree
    MakeViewableTree $canvas
    DN2Application::UpdateWindows $canvas
}

proc EditNonCompositeAttribs {canvas win} {
    # This routine updates the attributes for the noncomposite DN2 nodes.
    
    global qtd_text_value
    global nodetypes_with_value
    set nodetags [$canvas gettags selected]
    set nodeid [lindex $nodetags 1]
    set nodetype [TreeData::getnodetype $nodeid]
    set attribs [TreeData::getattriblist $nodeid]
    set textvalue [TreeData::gettextdata $nodeid]
    # loop through the attrib frames and grab the values of the attribute
    # name from the label and the attribute value from the entry
    if {![winfo exists $win.g]} {
	return
    }

    set attriblist ""
    set numattribs 0
    foreach i [grid slaves $win.g] {
	if {[regexp label* $i]} {incr numattribs}
    }
    for {set i 0} {$i < $numattribs} {incr i} {
	set attrib [string trimright [$win.g.label$i cget -text] :]
	if { ([lsearch -exact $nodetypes_with_value [string toupper $nodetype]] != -1)
	&& [string compare $attrib "Value"] == 0} {
	    # replace the data in the value variable with the text in the
	    # textwin.
	    TreeData::settextdata $nodeid [$win.g.text.entry dump -text 1.0 end]
	} else {
	    lappend attriblist [list $attrib [$win.g.entry$i get]]
	}
    }

    # set attriblist to be a list containing a null list
    # prevents a display problem that I ran into with Tree-4.2
    # If the tag list contains an empty list the drawing of the tree
    # containing that canvas objects goes nutty.  At the moment
    # it only seems to be a problem when the empty list is the second
    # tag.
    if {[llength $attriblist] == 0} {
	set attriblist {{}}
    }

    TreeData::setattriblist $nodeid $attriblist
}

proc EditCompositeAttribs {canvas win} {
    set nodeid [lindex [$canvas gettags selected] 1]
    set nodetype [TreeData::getnodetype $nodeid]

    if {[string compare $nodetype ANN-ATTR] == 0} {
	EditANN-ATTRAttribs $canvas $win $nodeid
    } elseif {[string compare $nodetype SE-APP] == 0} {
	# user can't change SE-APP so just return
	# EditSE-APPAttribs $canvas $nodeid
	return
    }
    return
}

proc EditANN-ATTRAttribs {canvas attribwin nodeid} {
    # Has up to 4 subnodes
    # ANN-TYPE - 0 or 1 allowed
    # ATTR-NAME - 0 or 1 allowed
    # ATTR-VALUE - 0 or 1 allowed
    # ATTR-TYPE - 0 or 1 allowed
    # none have attributes of their own
    # but all can have CDATA
    # 
    if {![winfo exists $attribwin.g]} {
	return
    }

    TreeData::prune_node $nodeid
    set numattribs 0
    foreach i [grid slaves $attribwin.g] {
	if {[regexp label* $i]} {incr numattribs}
    }
    for {set i 0} {$i < $numattribs} {incr i} {
	set nodename [string trimright [$attribwin.g.label$i cget -text] :]
	set newid [TreeData::makenewnode $canvas $nodename {} [$attribwin.g.text$i.entry dump -text 1.0 end]]
	TreeData::insert_rightmostchild $nodeid $newid
    }
    return
}

proc EditFULL-TERMAttribs {} {
    # populate later
}

proc EditSE-APPAttribs {} {
    # has 6 type of subnodes
    # SE-APP-HELP - 0 or 1 allowed
    # SE-APP-EXPL - 0 or 1 allowed
    # SE-APP-DOC-FREQ - 0 or 1 allowed
    # SE-APP-COLL-FREQ - 0 or 1 allowed
    # SE-APP-INTERNAL-ID - 0 or 1 allowed
    # SE-APP-MATCHES - 0 or more allowed
    # none have attributes of their own
    # but all can have CDATA
    # wait a minute a user can't change SE-APP can on view
    return
}

proc InsertDataIntoText {textwin data} {

    set datalength [llength $data]
    for {set i 1} {$i < $datalength} {incr i 3} {
	$textwin insert end [lindex $data $i]
    }
}

proc UpdateWindow {canvas} {
    # Check to see if there exists a .viewattribwin
    # skip everything else if there isn't one
    # first step is to clear out original window
    # The buttons will stay but anything else will be destroyed

    regsub -all {\.} $canvas {_} uniquifier
    set w .viewattribwin$uniquifier
    if {[winfo exists $w]} {
	# clear out data area
	catch {destroy $w.g}
	catch {destroy $w.label}
	$w.buttonframe.applybut configure -state normal

	if {[llength [$canvas find withtag selected]] == 0} {
	    # A window exists but there is no selected node
	    wm title $w "Attributes"
	    label $w.label -text "No Selected Node"
	    pack $w.label -side top
	    $w.buttonframe.applybut configure -state disabled
	} else {
	    # A window exists and there is a selected node.
	    # Check to see whether selected node is composite or not.
	    # Call the appropriate update routine.
	    if {[lsearch [$canvas gettags [$canvas find withtag selected]] noncomposite] == -1} {
		UpdateCompositeWindow $canvas $w
	    } else {
		UpdateNonCompositeWindow $canvas $w
	    }
	}
    }
}

proc UpdateNonCompositeWindow {canvas w} {
    # This routine updates a non-composite node attributes.
    # This is just a standard DN2 node.
    # loop through all of the attributes and construct a label containing the attribute name
    # and an entry box with the attribute value.  Place these name-value pairs into a 
    # grid.  Adding a test here to check for compositeness if it is composite call
    # update composite window.
    global nodetypes_with_value

    set added_attribute 0
    set added_textwin 0
    set nodetags [$canvas gettags selected]
    set nodeid [lindex $nodetags 1]
    set nodetype [TreeData::getnodetype $nodeid]
    set attribs [TreeData::getattriblist $nodeid]
    wm title $w "Attributes for $nodetype node"
    set numattribs [llength $attribs]
    set zero_attribs [expr ($numattribs == 1 && [llength [lindex $attribs 0]] == 0)]
    set g [frame $w.g]
    if {!$zero_attribs } {
	# There are some attribs so put them into $g
	grid rowconfig $g  0 -weight 1
	grid columnconfig $g 0 -weight 1
	for {set j 0} { $j < $numattribs } { incr j} {
	    #set f [frame $w.g.attrib$j]
	    label $g.label$j -width [string length [lindex [lindex $attribs $j] 0]] -text "[lindex [lindex $attribs $j] 0]:" -justify right
	    entry $g.entry$j -width [string length [lindex [lindex $attribs $j] 1]]
	    $g.entry$j insert end "[lindex [lindex $attribs $j] 1]"
	    grid configure $g.label$j -in $g -row $j -column 0 -sticky news
	    grid configure $g.entry$j -in $g -row $j -column 1 -sticky news
	    #pack $f.label $f.entry -side left -fill x
	}
	set added_attribute 1
    }
    if {([lsearch -exact $nodetypes_with_value [string toupper $nodetype]] != -1)} {
	# the nodetype is one which has value content so add a textwin to $g
	if {$added_attribute} {
	    # we can use the numattribs value as the index to the label
	    set label_id $numattribs
	} else {
	    # there were no attributes added to the window so the textwin will be only item
	    set label_id 0
	}
	label $g.label$label_id -width 5 -text "Value" -justify right
	# create text field for those nodes listed in nodetypes_with_value
	set textwin [frame $g.text]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 5 -wrap none
	InsertDataIntoText $textwin.entry [TreeData::gettextdata $nodeid]
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	grid columnconfig $textwin 0 -weight 1
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	grid configure $textwin -in $g -row $label_id -column 1 -sticky news
	grid configure $g.label$label_id -in $g -row $label_id -column 0 -sticky news
	set added_textwin 1
    }
    if {$added_attribute || $added_textwin} {
	pack $g -side top
    } else {
	label $w.label -text "This node has no attributes."
	pack $w.label -side top
    }
}

proc UpdateCompositeWindow {canvas w} {
    # Updating a composite node attribute window
    # This will determine the type of node and then will
    # call a routine for specifically handling that type
    # of composite node.
    set nodeid [lindex [$canvas gettags selected] 1]
    set nodetype [TreeData::getnodetype $nodeid]

    if {[string compare $nodetype ANN-ATTR] == 0} {
	UpdateANN-ATTRWindow $canvas $w $nodeid
    } elseif {[string compare $nodetype SE-APP] == 0} {
	UpdateSE-APPWindow $canvas $w $nodeid
    }
    #elseif {$nodetype == FULL-TERM} {
    #	UpdateFULL-TERMWindow $canvas $nodeid
    #} 
}

proc UpdateANN-ATTRWindow {canvas attribwin id} {
    # displays the editable fields for the ANN-ATTR
    # subnodes.

    #array set ann_attr {ANN-TYPE {} ATTR-NAME {} ATTR-TYPE {} ATTR-VALUE {}}
    array set ann_attr {}
    set current [TreeData::getchildid $id]
    if {$current != 0} {
	set ann_attr([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	set current [TreeData::getsiblingid $current]
	while {$current != 0} {
	    set ann_attr([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	    set current [TreeData::getsiblingid $current]
	}
    }

    set g [frame $attribwin.g]
    grid rowconfig $g 0 -weight 1
    grid columnconfig $g 0 -weight 1
    set cntr 0

    if {[info exists ann_attr(ANN-TYPE)]} {
	label $g.label$cntr -width 9 -text "ANN-TYPE:" -justify right
	set textwin [frame $g.text$cntr]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 2 -wrap none
	# put data into textwindow
	for {set i 1} {$i < [llength $ann_attr(ANN-TYPE)]} {incr i 3} {
	    $textwin.entry insert end [lindex $ann_attr(ANN-TYPE) $i]
	}
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	grid columnconfig $textwin 0 -weight 1
	# put text window and attached scrollbars into grid
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	# put label and text into grid
	grid configure $textwin -in $g -row $cntr -column 1 -sticky news
	grid configure $g.label$cntr -in $g -row $cntr -column 0 -sticky news
	incr cntr
	
    }

    if {[info exists ann_attr(ATTR-NAME)]} {
	label $g.label$cntr -width 10 -text "ATTR-NAME:" -justify right
	set textwin [frame $g.text$cntr]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 2 -wrap none
	# put data into textwindow
	for {set i 1} {$i < [llength $ann_attr(ATTR-NAME)]} {incr i 3} {
	    $textwin.entry insert end [lindex $ann_attr(ATTR-NAME) $i]
	}
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	grid columnconfig $textwin 0 -weight 1
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	grid configure $textwin -in $g -row $cntr -column 1 -sticky news
	grid configure $g.label$cntr -in $g -row $cntr -column 0 -sticky news
	incr cntr
    }

    if {[info exists ann_attr(ATTR-TYPE)]} {
	label $g.label$cntr -width 10 -text "ATTR-TYPE:" -justify right
	set textwin [frame $g.text$cntr]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 2 -wrap none
	# put data into textwindow
	for {set i 1} {$i < [llength $ann_attr(ATTR-TYPE)]} {incr i 3} {
	    $textwin.entry insert end [lindex $ann_attr(ATTR-TYPE) $i]
	}
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	grid columnconfig $textwin 0 -weight 1
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	grid configure $textwin -in $g -row $cntr -column 1 -sticky news
	grid configure $g.label$cntr -in $g -row $cntr -column 0 -sticky news
	incr cntr
    }

    if {[info exists ann_attr(ATTR-VALUE)]} {
	label $g.label$cntr -width 11 -text "ATTR-VALUE:" -justify right
	set textwin [frame $g.text$cntr]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 2 -wrap none
	# put data into textwindow
	for {set i 1} {$i < [llength $ann_attr(ATTR-VALUE)]} {incr i 3} {
	    $textwin.entry insert end [lindex $ann_attr(ATTR-VALUE) $i]
	}
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	grid columnconfig $textwin 0 -weight 1
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	grid configure $textwin -in $g -row $cntr -column 1 -sticky news
	grid configure $g.label$cntr -in $g -row $cntr -column 0 -sticky news
	incr cntr
    }

    if {$cntr > 0} {
	pack $g -side top
    } else {
	label $attribwin.label -text "This node has no attributes."
	pack $attribwin.label -side top
    }
    return
}

proc UpdateSE-APPWindow {canvas attribwin id} {
    # disables the apply button and displays the uneditable
    # fields of the SE-APP node and subnodes

    $attribwin.buttonframe.applybut configure -state disabled

    array set seapp {}
    set matchattribs {}
    set current [TreeData::getchildid $id]
    #puts "For $id"
    if {$current != 0} {
	# there are subnodes so display them
	lappend seapp([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	#puts "  child = $current"
	set current [TreeData::getsiblingid $current]
	while {$current != 0} {
	    #puts "  child = $current"
	    lappend seapp([TreeData::getnodetype $current]) [TreeData::gettextdata $current]
	    # put the MATCHES' attribs in an array for later display
	    if {[string compare [TreeData::getnodetype $current] "SE-APP-MATCHES"] == 0} {
		lappend matchattribs [TreeData::getattriblist $current]
	    }
	    set current [TreeData::getsiblingid $current]
	}
    }

    #puts [array get seapp]

    set g [frame $attribwin.g]
    grid rowconfig $g 0 -weight 1
    grid columnconfig $g 0 -weight 1

    set arrayptr [array startsearch seapp]
    set cntr 0
    set matchcntr 0
    while {[set element [array nextelement seapp $arrayptr]] != ""} {
	# do all other types
	case $element in {
	    SE-APP-HELP {set displayelement Help}
	    SE-APP-EXPL {set displayelement Explanation}
	    SE-APP-DOCFREQ {set displayelement "Doc Freq"}
	    SE-APP-COLLFREQ {set displayelement "Coll Freq"}
	    SE-APP-INTERNAL-ID {set displayelement "Internal ID"}
	    SE-APP-MATCHES {set displayelement "Matches"}
	}
	label $g.label$cntr -width [expr [string length $displayelement]+1] -text "$displayelement:" -justify right
	set textwin [frame $g.text$cntr]
	text $textwin.entry -xscrollcommand "$textwin.scrollx set" -yscrollcommand "$textwin.scrolly set" -setgrid true -height 2 -wrap none
	#puts $textwin
	for {set j 0} {$j < [llength $seapp($element)]} {incr j} {
	    if {[string compare $element SE-APP-MATCHES] == 0} {
		# display the attribs of the MATCH node before the MATCH info
		for {set i 0} {$i < 3} {incr i} {
		    $textwin.entry insert end [lindex [lindex [lindex $matchattribs $j] $i] 0]
		    $textwin.entry insert end "="
		    $textwin.entry insert end [lindex [lindex [lindex $matchattribs $j] $i] 1]
		    $textwin.entry insert end " "
		}
	    }
	    for {set i 1} {$i < [llength [lindex $seapp($element) $j]]} {incr i 3} {
		$textwin.entry insert end [lindex [lindex $seapp($element) $j] $i]
	    }
	}
	$textwin.entry configure -state disabled
	scrollbar $textwin.scrollx -orient horizon -command "$textwin.entry xview"
	scrollbar $textwin.scrolly -command "$textwin.entry yview"
	grid rowconfig $textwin 0 -weight 1
	# put text and scrollbars in its own grid
	grid configure $textwin.entry -in $textwin -row 0 -column 0 -sticky news
	grid configure $textwin.scrollx -in $textwin -row 1 -column 0 -sticky news
	grid configure $textwin.scrolly -in $textwin -row 0 -column 1 -sticky news
	# put label and textbox into grid
	grid configure $textwin -in $g -row $cntr -column 1 -sticky news
	grid configure $g.label$cntr -in $g -row $cntr -column -0 -sticky news

	incr cntr
    }
    array donesearch seapp $arrayptr
    if {$cntr > 0} {
	pack $g -side top
    } else {
	label $attribwin.label -text "This node has no attributes."
	pack $attribwin.label -side top
    }
    return
}


# Creates the .viewattribwin window and places
# pairs of label and entry for each attribute of a node.
# The value of the entry can be editted.
# Since this window is not modal it probably should be 
# called everytime that the selected node changes.
# 
# I declare that selecting one node while viewing another is
# equivalent to pressing cancel i.e. all changes to attributes
# are lost. 
proc CreateViewAttribWin {canvas} {

    regsub -all {\.} $canvas {_} uniquifier
    if {![winfo exists .viewattribwin$uniquifier]} {
	if {[llength [$canvas find withtag selected]] == 0} {
	    return
	}
	set w [toplevel .viewattribwin$uniquifier]
	
	frame $w.buttonframe
	button $w.buttonframe.newattribbut -text "Add Attribute" -command "AddAttrib::CreateAddAttribWin $canvas"
	button $w.buttonframe.applybut -text "Apply" -command "ViewAttribs::EditAttribs $canvas $w; ViewAttribs::FlashApply $w"
	button $w.buttonframe.dismissbut -text "Dismiss" -command "ViewAttribs::DestroyWin $canvas"
	pack $w.buttonframe.newattribbut $w.buttonframe.applybut $w.buttonframe.dismissbut -side left
	pack $w.buttonframe -side bottom -fill y  -expand 0
    } else {
	raise .viewattribwin$uniquifier
    }
    UpdateWindow $canvas


}

proc FlashApply {w} {
    # flash to apply button to give the user visual feedback that edit took place

    $w.buttonframe.applybut flash
}

proc DestroyWin {canvas} {
    regsub -all {\.} $canvas {_} uniquifier
    if {[winfo exists .viewattribwin$uniquifier]} {
	destroy .viewattribwin$uniquifier
    }
}

}
